From 0ad1db2e9212a4bafac04f94affaeecf0c9bb392 Mon Sep 17 00:00:00 2001
From: "Robert A. Burks" <rburksdev@gmail.com>
Date: Sun, 3 Mar 2024 15:50:55 -0500
Subject: [PATCH 01/31] Fix+ert segmentation fault in 'get-variable-watchers'
 (bug#00000)

Added check for valid symbol prior to passing it to
'SYMBOL_TRAPPED_WRITE_P'.

* src/data.c (Fget_variable_watchers):
'get-variable-watchers' will no longer cause a segmentation fault when
passed invalid arguments. (BUG#00000)
* test/src/data-tests.el (data-tests-get-variable-watchers-inputs):
Added ert for 'get-variable-watchers' bug. (BUG#00000)
---
 src/data.c             |  8 +++++---
 test/src/data-tests.el | 27 +++++++++++++++++++++++++++
 2 files changed, 32 insertions(+), 3 deletions(-)

diff --git a/src/data.c b/src/data.c
index c4b9cff8ae0..c42424497ad 100644
--- a/src/data.c
+++ b/src/data.c
@@ -1838,7 +1838,7 @@ SYMBOL (or its aliases) are set.  */)
   (Lisp_Object symbol, Lisp_Object watch_function)
 {
   symbol = Findirect_variable (symbol);
-  Lisp_Object watchers = Fget (symbol, Qwatchers);
+  Lisp_Object watchers = Fget (symbol, Qwatchers); /* CHECK_SYMBOL is in Fget */
   watchers = Fdelete (watch_function, watchers);
   if (NILP (watchers))
     {
@@ -1851,11 +1851,13 @@ SYMBOL (or its aliases) are set.  */)
 
 DEFUN ("get-variable-watchers", Fget_variable_watchers, Sget_variable_watchers,
        1, 1, 0,
-       doc: /* Return a list of SYMBOL's active watchers.  */)
+       doc: /* Return a list of functions actively watching SYMBOL (and its aliases).  */)
   (Lisp_Object symbol)
 {
+  symbol = Findirect_variable (symbol);
+  CHECK_SYMBOL (symbol); /* BUG#00000 Must guard SYMBOL_TRAPPED_WRITE_P */
   return (SYMBOL_TRAPPED_WRITE_P (symbol) == SYMBOL_TRAPPED_WRITE)
-    ? Fget (Findirect_variable (symbol), Qwatchers)
+    ? Fget (symbol , Qwatchers)
     : Qnil;
 }
 
diff --git a/test/src/data-tests.el b/test/src/data-tests.el
index a1959f62fd3..0778f8238ac 100644
--- a/test/src/data-tests.el
+++ b/test/src/data-tests.el
@@ -583,6 +583,33 @@ data-tests-local-variable-watchers
       (setq data-tests-lvar 6)
       (should (null watch-data)))))
 
+(ert-deftest data-tests-get-variable-watchers-inputs ()
+  (should-not (get-variable-watchers t))
+  (should-not (get-variable-watchers nil))
+  (should-not (get-variable-watchers :type))
+  (should-not (get-variable-watchers 'a-variable-that-does-not-exist))
+  (with-no-warnings
+    (should-error (get-variable-watchers a-variable-that-does-not-exist)))
+  ;; Without BUG#00000 fix, the tests below have been observed either
+  ;; failing or producing a segmentation fault.
+  (let ((foo1 "")
+        (foo2 "1")
+        (foo3 "load-path")
+        (foo4 "'load-path")
+        (foo5 "A longer string that is not a symbol name."))
+    (should-error (get-variable-watchers foo1))
+    (should-error (get-variable-watchers foo2))
+    (should-error (get-variable-watchers foo3))
+    (should-error (get-variable-watchers foo4))
+    (should-error (get-variable-watchers foo5)))
+  ;; Without BUG#00000 fix the tests below will segmentation fault
+  (let ((bar1 5)
+        (bar2 '("test"))
+        (bar3 [1 2 3]))
+    (should-error (get-variable-watchers bar1))
+    (should-error (get-variable-watchers bar2))
+    (should-error (get-variable-watchers bar3))))
+
 (ert-deftest data-tests-kill-all-local-variables () ;bug#30846
   (with-temp-buffer
     (setq-local data-tests-foo1 1)
-- 
2.34.1

